﻿EnableExplicit

XIncludeFile "Interface.pbf"

ImportC "-framework CoreServices"
  CFStringCreateWithCString(alloc, cStr.p-utf8, encoding = $8000100)
  MDItemCopyAttribute(item, name)
  MDItemCreate(allocator, path)
EndImport

Structure AudioItem
  title.s
  album.s
  duration.l
  fileName.s
  fileSize.q
EndStructure

Enumeration State
  #stopped
  #paused
  #playing
EndEnumeration

UsePNGImageDecoder()

Global kMDItemDurationSeconds.i = CFStringCreateWithCString(0, "kMDItemDurationSeconds")
Global kMDItemTitle.i = CFStringCreateWithCString(0, "kMDItemTitle")
Global kMDItemAlbum.i = CFStringCreateWithCString(0, "kMDItemAlbum")
Global kMDItemMusicalGenre.i = CFStringCreateWithCString(0, "kMDItemMusicalGenre")

Global.i NSSound, delegateClass, soundDelegate, time.d, State

Global NewList PlayList.AudioItem()

Declare ItemSelectedCallback()

Define.i i, Event, FileCount, Files.s, *Item.AudioItem, addDirectory


Procedure.s GetCFString(CFString.i)
  Protected CFStringLength.i = CFStringGetLength_(CFString)
  Protected Dim CharacterData.u(CFStringLength)
  CFStringGetCString_(CFString, @CharacterData(), CFStringLength + 1, #kCFStringEncodingUTF8)
  ProcedureReturn PeekS(@CharacterData(), -1, #PB_UTF8)
EndProcedure


Procedure GetMetaData(*Item.AudioItem, AudioFileName.s)
  Protected.i Path, MDItem, Attribute
  Path = CFStringCreateWithCString(0, AudioFileName)
  MDItem = MDItemCreate(0, Path)
  If MDItem
   
    ; get title
    Attribute = MDItemCopyAttribute(MDItem, kMDItemTitle)
    If Attribute
      *Item\title = GetCFString(Attribute)
      CFRelease_(Attribute)
    EndIf
   
    ; get album
    Attribute = MDItemCopyAttribute(MDItem, kMDItemAlbum)
    If Attribute
      *Item\album = GetCFString(Attribute)
      CFRelease_(Attribute)
    EndIf
   
    ; get duration
    Attribute = MDItemCopyAttribute(MDItem, kMDItemDurationSeconds)
    If Attribute
      CFNumberGetValue_(Attribute, #kCFNumberSInt32Type, @*Item\duration)
      CFRelease_(Attribute)
    EndIf
     
    CFRelease_(MDItem)
    CFRelease_(Path)
  EndIf
EndProcedure


Procedure AddAudio(List PlayList.AudioItem(), FileOrDirectory.s, Recurse = #False)
  
  Protected NewList Directory.s()
  Protected *Item.AudioItem
  Protected.i d, Size, DirectoryName.s, Extension.s
 
  Size = FileSize(FileOrDirectory)
  If Size = -2
    AddElement(Directory())
    Directory() = FileOrDirectory
  ElseIf Size > 0
    *Item = AddElement(PlayList())
    GetMetaData(*Item, FileOrDirectory)
    *Item\fileName = FileOrDirectory
    *Item\fileSize = Size
  EndIf
 
  While FirstElement(Directory())
    DirectoryName = Directory()
    DeleteElement(Directory())
    d = ExamineDirectory(#PB_Any, DirectoryName, "*.*")
    If d
      While NextDirectoryEntry(d)
        ; exclude files and directories starting with '.'
        If Asc(DirectoryEntryName(d)) <> '.'
          FileOrDirectory = DirectoryName + DirectoryEntryName(d)
          Extension = LCase(GetExtensionPart(FileOrDirectory))
          Size = FileSize(FileOrDirectory)
          If Size = -2
            ; exclude macOS apps which are actually directories
            If Recurse And Extension <> "app"
              AddElement(Directory())
              Directory() = FileOrDirectory + "/"
            EndIf
          ElseIf Size > 0
            If Extension = "mp3" Or Extension = "wav"
              *Item = AddElement(PlayList())
              GetMetaData(*Item, FileOrDirectory)
              *Item\fileName = FileOrDirectory
              *Item\fileSize = Size
            EndIf
          EndIf
        EndIf
      Wend
    EndIf
    FinishDirectory(d)
  Wend
 
EndProcedure


ProcedureC DidFinishPlayingCallback(obj,sel,sound,flag)
  
  Protected.i i
  If flag = #YES
    i = GetGadgetState(Playlist_0) + 1
    If i < CountGadgetItems(Playlist_0)
      SetGadgetState(Playlist_0, i)
      ItemSelectedCallback()
    Else
      SetGadgetState(Playlist_0, -1)
    EndIf     
  EndIf
EndProcedure


Procedure ItemSelectedCallback()
  
  Protected *Item.AudioItem
  *Item = GetGadgetItemData(Playlist_0, GetGadgetState(Playlist_0))
  If Not *Item = 0
    If NSSound
      CocoaMessage(0, NSSound, "stop")
      CocoaMessage(0, NSSound, "release")
    EndIf
    NSSound = CocoaMessage(0, CocoaMessage(0, 0, "NSSound alloc"),
                           "initWithContentsOfFile:$", @*Item\fileName, "byReference:", #YES)
    CocoaMessage(0, NSSound, "setDelegate:", soundDelegate)
   
    SetGadgetState(Img_Window_0_1, ImageID(Img_Window_0_7))
    SetGadgetAttribute(7, #PB_ProgressBar_Maximum, *Item\duration)
    
    CocoaMessage(0, NSSound, "play")
 
    Volume = GetGadgetState(TrackBar_0)/50
    CocoaMessage(0, NSSound, "setVolume:@", @Volume)
    State = #playing
  EndIf
EndProcedure


Procedure RefreshList()
  
  Protected s.s, i.i, *Item.AudioItem
  ClearGadgetItems(Playlist_0)
  ForEach Playlist()
    *Item = Playlist()
    s.s = *Item\title + #LF$
    s   + *Item\album + #LF$
    s   + FormatDate("%hh:%ii:%ss", *Item\duration) + #LF$
    s   + GetFilePart(*Item\fileName) + #LF$
    s   + FormatNumber(*Item\fileSize * 1e-6, 2) + #LF$
    AddGadgetItem(Playlist_0, i, s)
    SetGadgetItemData(Playlist_0, i, *Item)
    i + 1
  Next
  SetGadgetState(Playlist_0, -1)
EndProcedure


Procedure ReadDirectory(folderPath$)

  If ExamineDirectory(0, folderPath$, "*.mp3")
    
    ClearList(Playlist())
    
    While NextDirectoryEntry(0)
      If DirectoryEntryType(0) = #PB_DirectoryEntry_File
        AddAudio(Playlist(), folderPath$ + DirectoryEntryName(0), #False)
       
      EndIf
    Wend

    FinishDirectory(0)
    RefreshList()
  EndIf
 
EndProcedure


Procedure Sound_SetVolume(SoundObject, Volume.f)
  CocoaMessage(0, SoundObject, "setVolume:@", @Volume)
EndProcedure


delegateClass = objc_allocateClassPair_(objc_getClass_("NSObject"), "SoundDelegateClass",0)
class_addMethod_(delegateClass,sel_registerName_("sound:didFinishPlaying:"), @DidFinishPlayingCallback(), "v@:@c")
objc_registerClassPair_(delegateClass)
soundDelegate = class_createInstance_(delegateClass, 0)

OpenWindow_0()
AddWindowTimer(Window_0, 0, 100)

BindEvent(#PB_Event_Gadget, @ItemSelectedCallback(), ExplorerTree_0, PlayList_0, #PB_EventType_LeftClick)

State = #stopped

; >>> Main loop <<<

Repeat
  Event = WaitWindowEvent()
  Select Event
      
    Case #PB_Event_Timer
      If State = #playing
        CocoaMessage(@time, NSSound, "currentTime")
        SetGadgetState(7, PeekD(@time))
        SetGadgetText(10, FormatDate("%ii:%ss", time))
      EndIf
      
    Case  #PB_Event_GadgetDrop
      Files = EventDropFiles()
      FileCount = CountString(files, #LF$) + 1
      For i = 1 To FileCount
        AddAudio(PlayList(), StringField(Files, i, #LF$), #False)
      Next
      RefreshList()
      
    Case #PB_Event_Gadget
      If EventGadget() = 1
        Select EventType()
  
          Case #PB_EventType_LeftClick
            
            ReadDirectory(GetGadgetText(1))
        EndSelect
      Else
        Select EventType()
            
          Case #PB_EventType_LeftClick
            Select EventGadget()
                
              Case 2 ; Backward
                i = GetGadgetState(Playlist_0) - 1
                If i > -1
                  SetGadgetState(Playlist_0, i)
                  ItemSelectedCallback()
                EndIf
                
              Case 3 ; Play
                If State = #paused
                  SetGadgetState(Img_Window_0_1, ImageID(Img_Window_0_7))
                  CocoaMessage(0, NSSound, "resume")
                  State = #playing
                ElseIf State = #Playing
                  SetGadgetState(Img_Window_0_7, ImageID(Img_Window_0_1))
                  CocoaMessage(0, NSSound, "pause")
                  State = #paused
                EndIf
                
              Case 4 ; Forward
                i = GetGadgetState(Playlist_0) + 1
                If i < CountGadgetItems(Playlist_0)
                  SetGadgetState(Playlist_0, i)
                  ItemSelectedCallback()
                EndIf
                
              Case 5 ; Stop
                If State = #playing
                  CocoaMessage(0, NSSound, "stop")
                  SetGadgetState(Img_Window_0_7, ImageID(Img_Window_0_1))
                  State = #stopped
                EndIf
                
              Case 6 ; Hide
                
              Case 11 ; Volume
                If NSSound = 0
                  SetGadgetState(11, 45)
                Else
                  Sound_SetVolume(NSSound, GetGadgetState(11)/50)
                EndIf
            EndSelect
          EndSelect
        EndIf
    EndSelect
Until Event = #PB_Event_CloseWindow


If NSSound
  CocoaMessage(0, NSSound, "stop")
  CocoaMessage(0, NSSound, "release")
EndIf

; Procedure Window_0_Events(event)
;   Select event
;     Case #PB_Event_SizeWindow
;       ResizeGadgetsWindow_0()
;     Case #PB_Event_CloseWindow
;       ProcedureReturn #False
; 
;     Case #PB_Event_Menu
;       Select EventMenu()
;       EndSelect
; 
;     Case #PB_Event_Gadget
;       Select EventGadget()
;       EndSelect
;   EndSelect
;   ProcedureReturn #True
; EndProcedure
; IDE Options = PureBasic 5.60 (MacOS X - x64)
; CursorPosition = 224
; Folding = A9
; EnableXP
; Executable = Mp3 Player 0.1.app